#define WATER_PARALAX

float waterH(vec2 posxz, float waveM, float waveZ, float waveS) {
	
	float timer = 0.75;
	mat2 rotation = mat2(
	vec2(cos(timer), -sin(timer)),
	vec2(sin(timer), cos(timer))
	);
	posxz = posxz * vec2(0.5,1.5) * rotation;
	
	vec2 movement = vec2(-abs(frameTimeCounter/2000),abs(frameTimeCounter/2000)) * waveM;

	vec2 coord  = cos((posxz / 600 * waveZ)- movement / 8.0);
	vec2 coord1 = ((posxz / 599 * waveZ)- movement / 4.0);
	vec2 coord2 = ((posxz / 598 * waveZ)+ movement / 2.0);
	vec2 coord3 = ((posxz / 597 * waveZ)- movement / 1.0);
	
	float noise  = texture2D(noisetex,fract(coord.xy*0.5)).x/1.0*waveS;
		  noise *= texture2D(noisetex,fract(coord1.xy*0.75)).x/0.7*waveS;
		  noise *= texture2D(noisetex,fract(coord2.xy*2.0)).x/2.0*waveS;
		  noise += texture2D(noisetex,fract(coord3.xy*0.5)).x/0.8*waveS;

	return pow(noise, 2.0) / 2.0;
}

vec2 paralaxCoords(vec3 pos, vec3 tangentVector, float istransparent) {
	float waveZ = mix(0.6,0.1,istransparent);
	float waveM = mix(4.0,0.0,istransparent);
	float waveS = mix(1.0,0.5,istransparent);

	float waterHeight = waterH(pos.xz - pos.y, waveM, waveZ, waveS);
	
	vec3 paralaxCoord = vec3(0.0, 0.0, 1.0);
	vec3 stepSize = vec3(waveS, waveS, 1.0) * 0.6;
	vec3 step = tangentVector * stepSize;
	
	for (int i = 0; waterHeight < paralaxCoord.z && i < 20; i++) {
		paralaxCoord.xy = mix(paralaxCoord.xy, paralaxCoord.xy + step.xy, clamp((paralaxCoord.z - waterHeight) / (stepSize.z * 0.2f / (-tangentVector.z + 0.05f)), 0.0f, 1.0f));
		paralaxCoord.z += step.z;
		vec3 paralaxPosition = pos + vec3(paralaxCoord.x, 0.0f, paralaxCoord.y);
		waterHeight = waterH(paralaxPosition.xz - paralaxPosition.y, waveM, waveZ, waveS);
	}
	pos += vec3(paralaxCoord.x, 0.0f, paralaxCoord.y);
	return pos.xz - pos.y;
}

vec3 getWaveHeight(vec3 posxz, float iswater, float istransparent, vec3 tangentVector){

	vec2 coord = posxz.xz - posxz.y;
	
	#if defined WATER_PARALAX
		coord = paralaxCoords(posxz, tangentVector, istransparent);
	#endif
	
		float deltaPos = 0.22;
		
		float waveZ = mix(0.6,3.0,istransparent);
		float waveM = mix(4.0,0.0,istransparent);
		float waveS = mix(1.0,0.2,istransparent);
		
		float h0 = waterH(coord, waveM, waveZ, waveS);
		float h1 = waterH(coord + vec2(deltaPos,0.0), waveM, waveZ, waveS);
		float h2 = waterH(coord + vec2(-deltaPos,0.0), waveM, waveZ, waveS);
		float h3 = waterH(coord + vec2(0.0,deltaPos), waveM, waveZ, waveS);
		float h4 = waterH(coord + vec2(0.0,-deltaPos), waveM, waveZ, waveS);

		float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
		float yDelta = ((h3-h0)+(h0-h4))/deltaPos;

		vec3 wave = normalize(vec3(xDelta,yDelta,1.0-pow(abs(xDelta+yDelta),2.0)));

		return wave;
}

vec3 getWaveHeightFlat(vec3 posxz, float iswater, float istransparent){

	vec2 coord = posxz.xz - posxz.y;
	
		float deltaPos = 0.22;
		
		float waveZ = mix(0.6,3.0,istransparent);
		float waveM = mix(4.0,0.0,istransparent);
		float waveS = mix(1.0,0.2,istransparent);
		
		float h0 = waterH(coord, waveM, waveZ, waveS);
		float h1 = waterH(coord + vec2(deltaPos,0.0), waveM, waveZ, waveS);
		float h2 = waterH(coord + vec2(-deltaPos,0.0), waveM, waveZ, waveS);
		float h3 = waterH(coord + vec2(0.0,deltaPos), waveM, waveZ, waveS);
		float h4 = waterH(coord + vec2(0.0,-deltaPos), waveM, waveZ, waveS);

		float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
		float yDelta = ((h3-h0)+(h0-h4))/deltaPos;

		vec3 wave = normalize(vec3(xDelta,yDelta,1.0-pow(abs(xDelta+yDelta),2.0)));

		return wave;
}